﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows;

using Telerik.Windows.Controls.DragDrop;
using System.Collections;
using System.Collections.ObjectModel;

namespace SilverlightApplication15
{
	public partial class Page : UserControl
	{
		public Page()
		{
			InitializeComponent();
			dragDropControl1.ItemsSource = GetItems("ListBox1");
			dragDropControl2.ItemsSource = GetItems("ListBox2");
		}

		private ObservableCollection<String> GetItems(string itemName)
		{
			ObservableCollection<string> target = new ObservableCollection<string>();
			for (int i = 0; i < 5; i++)
			{
				target.Add(String.Format(itemName + ".Item" + i));
			}
			return target;
		}
	}

	public class DragDropListBox : ListBox
	{
		public DragDropListBox()
		{
			this.AddHandler(RadDragAndDropManager.DragQueryEvent, new EventHandler<DragDropQueryEventArgs>(OnDragQuery));
			this.AddHandler(RadDragAndDropManager.DropQueryEvent, new EventHandler<DragDropQueryEventArgs>(OnDropQuery));
			this.AddHandler(RadDragAndDropManager.DropInfoEvent, new EventHandler<DragDropEventArgs>(OnDropInfo));
			this.AddHandler(RadDragAndDropManager.DragInfoEvent, new EventHandler<DragDropEventArgs>(OnDragInfo));
	
			// Enable dropping
			RadDragAndDropManager.SetAllowDrop(this, true);
		}

		// OnDragQuery event handler
		private void OnDragQuery(object sender, DragDropQueryEventArgs e)
		{
			// An object is about the be dragged
			if (e.Options.Status == DragStatus.DragQuery)
			{
				e.Options.DragCue = RadDragAndDropManager.GenerateVisualCue(new Rectangle() { Width = 10, Height = 10 });
				
				// Set result to true
				e.QueryResult = true;
				// Get the string representation of the item that is being dragged
				string draggedItem = (string)(e.Options.Source as System.Windows.Controls.ListBoxItem).Content;
				// Put a new instance of the DragRequest class within Payload
				// The DragRequest instance contains the string representation of the object being dragged,
				// as well as the origin the the object
				e.Options.Payload = new DragRequest() { Item = draggedItem, ItemsHost = this };
				
				e.Source = this;
				e.Handled = true;
			}

			// Ask if it is OK to drop the target at the particular location
			if (e.Options.Status == DragStatus.DropSourceQuery)
			{
				e.QueryResult = true;
				e.Handled = true;
			}
		}

		// OnDropQuery event handler
		private void OnDropQuery(object sender, DragDropQueryEventArgs e)
		{
			// Ask the destination if dropping is possible
			if (e.Options.Status == DragStatus.DropDestinationQuery)
			{
				// Check whether the source and the target are not the same
				if (this != e.Options.Source)
				{
					DragRequest request = e.Options.Payload as DragRequest;
					// Check whether the control where the dragged object comes from is 
					// different than the one we are dropping the object onto.
					e.QueryResult = (request.ItemsHost != this);
					if (e.QueryResult == true)
					{
						e.Handled = true;
					}
				}
			}
		}

		// OnDropInfo event handler
		private void OnDropInfo(object sender, DragDropEventArgs e)
		{
			// Drop has completed successfully
			if (e.Options.Status == DragStatus.DropComplete)
			{
				// Add the dropped object to its new parent
				DragRequest request = e.Options.Payload as DragRequest;
				(this.ItemsSource as IList).Add(request.Item);
				e.Handled = true;
			}
		}

		// OnDragInfo event handler
		private void OnDragInfo(object sender, DragDropEventArgs e)
		{	
			// Drag has completed successfully
			if (e.Options.Status == DragStatus.DragComplete)
			{
				// Remove the dragged object from its original parent
				DragRequest request = e.Options.Payload as DragRequest;
				(this.ItemsSource as IList).Remove(request.Item);
				e.Handled = true;
			}
		}

		// Enable dragging of the children
		protected override void PrepareContainerForItemOverride(DependencyObject element, object item)
		{
			base.PrepareContainerForItemOverride(element, item);
			RadDragAndDropManager.SetAllowDrag(element, true);
		}
	}
	
	// This class will be placed within the Payload property during the drag/drop operation
	public class DragRequest
	{
		public String Item { get; set; }
		public ListBox ItemsHost { get; set; }
	}
}
