﻿using System;
using System.Windows;
using System.Windows.Controls;
using Telerik.Windows;
using Telerik.Windows.Controls.DragDrop;
using System.Collections.ObjectModel;
using System.Collections;

namespace SilverlightApplication16
{
	public partial class Page : UserControl
	{
		public Page()
		{
			InitializeComponent();

			groupA.ItemsSource = GetChampionsLeagueGroup(Group.A);
			groupB.ItemsSource = GetChampionsLeagueGroup(Group.B);
		}

		public ObservableCollection<Team> GetChampionsLeagueGroup(Group group)
		{
			ObservableCollection<Team> target = new ObservableCollection<Team>();

			if (group == Group.A)
			{
				target.Add(new Team("Chelsea", @"logos/chelsea.png"));
				target.Add(new Team("CFR Cluj-Napoca", @"logos/cluj.png"));
				target.Add(new Team("Roma", @"logos/roma.png"));
				target.Add(new Team("Bordeaux", @"logos/bordeaux.png"));
			}
			if (group == Group.B)
			{
				target.Add(new Team("Inter Milan", @"logos/inter.png"));
				target.Add(new Team("Anorthosis", @"logos/anorthosis.png"));
				target.Add(new Team("Werder Bremen", @"logos/werder.png"));
				target.Add(new Team("Panathinaikos", @"logos/panatinaikos.png"));
			}
			return target;
		}
	}

	public class DraggableListBox : ListBox
	{
		public DraggableListBox()
		{
			this.AddHandler(RadDragAndDropManager.DragQueryEvent, new EventHandler<DragDropQueryEventArgs>(OnDragQuery));
			this.AddHandler(RadDragAndDropManager.DropQueryEvent, new EventHandler<DragDropQueryEventArgs>(OnDropQuery));
			this.AddHandler(RadDragAndDropManager.DropInfoEvent, new EventHandler<DragDropEventArgs>(OnDropInfo));
			this.AddHandler(RadDragAndDropManager.DragInfoEvent, new EventHandler<DragDropEventArgs>(OnDragInfo));

			RadDragAndDropManager.SetAllowDrop(this, true);
		}

		private void OnDragQuery(object sender, DragDropQueryEventArgs e)
		{
			DraggableListBox box = sender as DraggableListBox;

			if (e.Options.Status == DragStatus.DragQuery)
			{
				Page page = new Page();
				
				e.QueryResult = true;

				ContentControl cue = new ContentControl();
				cue.ContentTemplate = page.Resources["ApplicationDragTemplate"] as DataTemplate;
				cue.Content = box.SelectedItem;
				e.Options.DragCue = cue;

				e.Options.Payload = new DragRequest() { Item = box.SelectedItem as Team, ItemsHost = this };
				e.Source = this;
				e.Handled = true;
			}

			if (e.Options.Status == DragStatus.DropSourceQuery)
			{
				e.QueryResult = true;
				e.Handled = true;
			}
		}
		private void OnDropQuery(object sender, DragDropQueryEventArgs e)
		{
			if (e.Options.Status == DragStatus.DropDestinationQuery)
			{
				if (this != e.Options.Source)
				{
					DragRequest request = e.Options.Payload as DragRequest;

					e.QueryResult = request.ItemsHost != this;

					if (e.QueryResult == true)
					{
						e.Handled = true;
					}
				}
			}
		}
		private void OnDropInfo(object sender, DragDropEventArgs e)
		{
			if (e.Options.Status == DragStatus.DropComplete)
			{
				DragRequest request = e.Options.Payload as DragRequest;
				(this.ItemsSource as IList).Add(request.Item);
				e.Handled = true;
			}
		}
		private void OnDragInfo(object sender, DragDropEventArgs e)
		{
			if (e.Options.Status == DragStatus.DragComplete)
			{
				DragRequest request = e.Options.Payload as DragRequest;
				(this.ItemsSource as IList).Remove(request.Item);
				e.Handled = true;
			}
		}

		protected override void PrepareContainerForItemOverride(DependencyObject element, object item)
		{
			base.PrepareContainerForItemOverride(element, item);
			RadDragAndDropManager.SetAllowDrag(element, true);
		}
	}

	public class DragRequest
	{
		public Team Item
		{
			get;
			set;
		}
		public DraggableListBox ItemsHost
		{
			get;
			set;
		}
	}

	public class Team
	{
		public Team() { }
		public Team(string name, string iconPath)
		{
			this.IconPath = iconPath;
			this.Name = name;
		}
		public string IconPath
		{
			get;
			set;
		}
		public string Name
		{
			get;
			set;
		}
	}

	public enum Group
	{
		A,
		B
	}
}
